import numpy as np
import matplotlib.pyplot as plt
from .visualize import ax_show
from ..process.calibration import get_Qvector_from_Rvector,get_Rvector_from_Qvector

def show_RQ(realspace_image, realspace_pdict,
            diffractionspace_image, diffractionspace_pdict,
            figsize=(12,6),returnfig=False):
    """
    Shows side-by-side real/reciprocal space images.

    Accepts:
        realspace_image             (2D array)
        diffractionspace_image      (2D array)
        realspace_pdict             (dictionary) arguments and values to pass
                                    to the show() fn for the real space image
        diffractionspace_pdict      (dictionary)
    """
    fig,(ax1,ax2) = plt.subplots(1,2,figsize=figsize)
    ax_show(realspace_image,ax1,**realspace_pdict)
    ax_show(diffractionspace_image,ax2,**diffractionspace_pdict)
    if not returnfig:
        plt.show()
        return
    else:
        return fig,(ax1,ax2)

def show_RQ_vector(realspace_image, realspace_pdict,
                   diffractionspace_image, diffractionspace_pdict,
                   vx,vy,vlength_R,vlength_Q,x0_R,y0_R,x0_Q,y0_Q,
                   QR_rotation,vector_space='R',
                   width_R=1,color_R='r',
                   width_Q=1,color_Q='r',
                   figsize=(12,6),returnfig=False):
    """
    Shows side-by-side real/reciprocal space images with a vector
    overlaid in each showing corresponding directions.

    Accepts:
        realspace_image             (2D array)
        diffractionspace_image      (2D array)
        realspace_pdict             (dictionary) arguments and values to pass
                                    to the show() fn for the real space image
        diffractionspace_pdict      (dictionary)
        vx,vy                       (numbers) x,y components of the vector
                                    in either real or diffraction space,
                                    depending on the value of vector_space.
                                    Note (vx,vy) is used for the orientation
                                    only - the two vectors are normalized
                                    and rescaled by
        vlength_R,vlenght_Q         (number) the vector length in each
                                    space, in pixels
        x0_R,y0_R,x0_Q,y0_Q         (numbers) the origins / vector tail positions
        QR_rotatation               (number) the offset angle between real and
                                    diffraction space. Specifically, this is
                                    the counterclockwise rotation of real space
                                    with respect to diffraction space. In degrees.
        vector_space                (string) must be 'R' or 'Q'. Specifies
                                    whether the (vx,vy) values passed to this
                                    function describes a real or diffracation
                                    space vector.
    """
    assert(vector_space in ('R','Q'))
    fig,(ax1,ax2) = show_RQ(realspace_image, realspace_pdict,
                            diffractionspace_image, diffractionspace_pdict,
                            figsize=figsize,returnfig=True)
    # Get pair of corresponding Q/R unit vectors
    if vector_space=='R':
        vx_R,vy_R,vx_Q,vy_Q = get_Qvector_from_Rvector(vx,vy,QR_rotation)
    else:
        vx_R,vy_R,vx_Q,vy_Q = get_Rvector_from_Qvector(vx,vy,QR_rotation)
    # Scale magnitude
    vx_R,vy_R = vx_R*vlength_R,vy_R*vlength_R
    vx_Q,vy_Q = vx_Q*vlength_Q,vy_Q*vlength_Q

    ax1.arrow(y0_R,x0_R,vy_R,vx_R,color=color_R,width=width_R,length_includes_head=True)
    ax2.arrow(y0_Q,x0_Q,vy_Q,vx_Q,color=color_Q,width=width_Q,length_includes_head=True)
    if not returnfig:
        plt.show()
        return
    else:
        return fig,(ax1,ax2)

def show_RQ_vectors(realspace_image, realspace_pdict,
                    diffractionspace_image, diffractionspace_pdict,
                    vx,vy,vlength_R,vlength_Q,x0_R,y0_R,x0_Q,y0_Q,
                    QR_rotation,vector_space='R',
                    width_R=1,color_R='r',
                    width_Q=1,color_Q='r',
                    figsize=(12,6),returnfig=False):
    """
    Shows side-by-side real/reciprocal space images with multiple vectors
    overlaid in each showing corresponding directions.

    Accepts:
        realspace_image             (2D array)
        diffractionspace_image      (2D array)
        realspace_pdict             (dictionary) arguments and values to pass
                                    to the show() fn for the real space image
        diffractionspace_pdict      (dictionary)
        vx,vy                       (1D arrays) x,y components of the vectors
                                    in either real or diffraction space,
                                    depending on the value of vector_space.
                                    Note (vx,vy) is used for the orientations
                                    only - the vectors are normalized
                                    and rescaled by
        vlength_R,vlenght_Q         (number or 1D arrays) the vector length in each
                                    space, in pixels
        x0_R,y0_R,x0_Q,y0_Q         (number) the origins / vector tail positions
        QR_rotatation               (number) the offset angle between real and
                                    diffraction space. Specifically, this is
                                    the counterclockwise rotation of real space
                                    with respect to diffraction space. In degrees.
        vector_space                (string) must be 'R' or 'Q'. Specifies
                                    whether the (vx,vy) values passed to this
                                    function describes a real or diffracation
                                    space vector.
    """
    assert(vector_space in ('R','Q'))
    assert(len(vx)==len(vy))
    if isinstance(color_R,tuple) or isinstance(color_R,list):
        assert(len(color_R)==len(vx))
    if isinstance(color_Q,tuple) or isinstance(color_Q,list):
        assert(len(color_Q)==len(vx))

    fig,(ax1,ax2) = show_RQ(realspace_image, realspace_pdict,
                            diffractionspace_image, diffractionspace_pdict,
                            figsize=figsize,returnfig=True)
    # Get pair of corresponding Q/R unit vectors
    if vector_space=='R':
        vx_R,vy_R,vx_Q,vy_Q = get_Qvector_from_Rvector(vx,vy,QR_rotation)
    else:
        vx_R,vy_R,vx_Q,vy_Q = get_Rvector_from_Qvector(vx,vy,QR_rotation)
    # Scale magnitude
    vx_R,vy_R = vx_R*vlength_R,vy_R*vlength_R
    vx_Q,vy_Q = vx_Q*vlength_Q,vy_Q*vlength_Q

    for i in range(len(vx_R)):
        _vx_R,_vy_R,_vx_Q,_vy_Q = vx_R[i],vy_R[i],vx_Q[i],vy_Q[i]
        if isinstance(color_R,tuple) or isinstance(color_R,list):
            _color_R = color_R[i]
        else:
            _color_R = color_R
        if isinstance(color_Q,tuple) or isinstance(color_Q,list):
            _color_Q = color_Q[i]
        else:
            _color_Q = color_Q
        ax1.arrow(y0_R,x0_R,_vy_R,_vx_R,color=_color_R,width=width_R,length_includes_head=True)
        ax2.arrow(y0_Q,x0_Q,_vy_Q,_vx_Q,color=_color_Q,width=width_Q,length_includes_head=True)
    if not returnfig:
        plt.show()
        return
    else:
        return fig,(ax1,ax2)

def show_RQ_axes(realspace_image, realspace_pdict,
                 diffractionspace_image, diffractionspace_pdict,
                 vx,vy,vlength_R,vlength_Q,x0_R,y0_R,x0_Q,y0_Q,
                 QR_rotation,vector_space='R',
                 width_R=1,color_R='r',width_Q=1,color_Q='r',
                 labelaxes=True,labelsize=12,labelcolor_R='r',labelcolor_Q='r',
                 labelsize_R=12,labelsize_Q=12,figsize=(12,6),returnfig=False):
    """
    Shows side-by-side real/reciprocal space images with a set of corresponding
    coordinate axes overlaid in each.  (vx,vy) specifies the x-axis, and the y-axis
    is rotated 90 degrees counterclockwise in reciprocal space (relevant in case of
    an R/Q transposition).

    Accepts:
        realspace_image             (2D array)
        diffractionspace_image      (2D array)
        realspace_pdict             (dictionary) arguments and values to pass
                                    to the show() fn for the real space image
        diffractionspace_pdict      (dictionary)
        vx,vy                       (numbers) x,y components of the x-axis
                                    in either real or diffraction space,
                                    depending on the value of vector_space.
                                    Note (vx,vy) is used for the orientation
                                    only - the vectors are normalized
                                    and rescaled by
        vlength_R,vlenght_Q         (number or 1D arrays) the vector length in each
                                    space, in pixels
        x0_R,y0_R,x0_Q,y0_Q         (number) the origins / vector tail positions
        QR_rotatation               (number) the offset angle between real and
                                    diffraction space. Specifically, this is
                                    the counterclockwise rotation of real space
                                    with respect to diffraction space. In degrees.
        vector_space                (string) must be 'R' or 'Q'. Specifies
                                    whether the (vx,vy) values passed to this
                                    function describes a real or diffracation
                                    space vector.
    """
    assert(vector_space in ('R','Q'))
    fig,(ax1,ax2) = show_RQ(realspace_image, realspace_pdict,
                            diffractionspace_image, diffractionspace_pdict,
                            figsize=figsize,returnfig=True)
    # Get the x-axes unit vectors in Q and R
    if vector_space=='R':
        vx_Rxaxis,vy_Rxaxis,vx_Qxaxis,vy_Qxaxis = get_Qvector_from_Rvector(vx,vy,QR_rotation)
    else:
        vx_Rxaxis,vy_Rxaxis,vx_Qxaxis,vy_Qxaxis = get_Rvector_from_Qvector(vx,vy,QR_rotation)
    # Get the y-axes
    vx_Qyaxis,vy_Qyaxis = -vy_Qxaxis,vx_Qxaxis
    vx_Ryaxis,vy_Ryaxis,_,_ = get_Rvector_from_Qvector(vx_Qyaxis,vy_Qyaxis,QR_rotation)
    # Scale magnitudes
    vx_Rxaxis,vy_Rxaxis = vx_Rxaxis*vlength_R,vy_Rxaxis*vlength_R
    vx_Qxaxis,vy_Qxaxis = vx_Qxaxis*vlength_Q,vy_Qxaxis*vlength_Q
    vx_Ryaxis,vy_Ryaxis = vx_Ryaxis*vlength_R,vy_Ryaxis*vlength_R
    vx_Qyaxis,vy_Qyaxis = vx_Qyaxis*vlength_Q,vy_Qyaxis*vlength_Q
    # Plot vectors
    ax1.arrow(y0_R,x0_R,vy_Rxaxis,vx_Rxaxis,color=color_R,width=width_R,length_includes_head=True)
    ax1.arrow(y0_R,x0_R,vy_Ryaxis,vx_Ryaxis,color=color_R,width=width_R,length_includes_head=True)
    ax2.arrow(y0_Q,x0_Q,vy_Qxaxis,vx_Qxaxis,color=color_Q,width=width_Q,length_includes_head=True)
    ax2.arrow(y0_Q,x0_Q,vy_Qyaxis,vx_Qyaxis,color=color_Q,width=width_Q,length_includes_head=True)
    # Label axes:
    if labelaxes==True:
        Rxaxislabel_x = x0_R + 1.1*vx_Rxaxis
        Rxaxislabel_y = y0_R + vy_Rxaxis
        Ryaxislabel_x = x0_R + vx_Ryaxis
        Ryaxislabel_y = y0_R + 1.1*vy_Ryaxis
        Qxaxislabel_x = x0_Q + 1.1*vx_Qxaxis
        Qxaxislabel_y = y0_Q + vy_Qxaxis
        Qyaxislabel_x = x0_Q + vx_Qyaxis
        Qyaxislabel_y = y0_Q + 1.1*vy_Qyaxis
        ax1.text(Rxaxislabel_y,Rxaxislabel_x,'x',color=labelcolor_R,size=labelsize_R)
        ax1.text(Ryaxislabel_y,Ryaxislabel_x,'y',color=labelcolor_R,size=labelsize_R)
        ax2.text(Qxaxislabel_y,Qxaxislabel_x,'x',color=labelcolor_Q,size=labelsize_Q)
        ax2.text(Qyaxislabel_y,Qyaxislabel_x,'y',color=labelcolor_Q,size=labelsize_Q)
    if not returnfig:
        plt.show()
        return
    else:
        return fig,(ax1,ax2)

def show_strain():

    return

